package com.example.tfat.myapplication

import android.app.Activity
import android.app.ProgressDialog
import android.content.DialogInterface
import android.content.Intent
import android.graphics.Bitmap
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.DisplayMetrics
import android.view.*
import android.widget.PopupWindow
import android.widget.SeekBar
import com.bomostory.sceneeditmodule.*
import com.bomostory.sceneeditmodule.basicdata.*
import com.bomostory.sceneeditmodule.navigationbar.actor.SelectActorView
import com.bomostory.sceneeditmodule.navigationbar.brush.BrushView
import com.bomostory.sceneeditmodule.navigationbar.dialogue.DialogueView
import com.bomostory.sceneeditmodule.screen.movie.MovieEditActivity
import com.bomostory.sceneeditmodule.screen.view.EditActorView
import com.bomostory.sceneeditmodule.screen.view.LayerView
import com.bomostory.sceneeditmodule.screen.view.OnTouchBoMoSceneListener
import com.bomostory.sceneeditmodule.screen.view.OnTouchSceneListener
import com.bomostory.sceneeditmodule.utils.FileUtils
import com.example.tfat.myapplication.navigationbar.EditSceneView
import com.example.tfat.myapplication.navigationbar.NavigationBarView
import com.example.tfat.myapplication.navigationbar.RecordFinishView
import com.example.tfat.myapplication.navigationbar.actor.ActorAdapter
import com.example.tfat.myapplication.navigationbar.actor.LayerManagementDialog
import com.example.tfat.myapplication.navigationbar.scene.AddSceneAdapter
import com.example.tfat.myapplication.navigationbar.scene.AddSceneView
import com.example.tfat.myapplication.navigationbar.scene.ControlSceneView
import com.example.tfat.myapplication.navigationbar.scene.SceneAdapter
import com.google.gson.Gson
import kotlinx.android.synthetic.main.activity_scene_edit.*
import kotlinx.android.synthetic.main.control_actor_dialog.view.*
import android.widget.RadioGroup
import com.bomostory.sceneeditmodule.EditTextDialog
import kotlinx.android.synthetic.main.popupview_opacity_dialog.view.*
import kotlinx.android.synthetic.main.view_control_dialogue_dialog.view.*
import com.bomostory.sceneeditmodule.DialogueColorData
import com.bomostory.sceneeditmodule.SceneDrawer
import com.bomostory.sceneeditmodule.navigationbar.actor.ObjectView
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.actor_select_view.view.*
import kotlinx.android.synthetic.main.layer_management_fragment.*
import kotlinx.android.synthetic.main.navigation_bar_view.view.*
import kotlinx.android.synthetic.main.popupview_color_dialog.view.*
import kotlinx.android.synthetic.main.popupview_setting.view.*
import kotlinx.android.synthetic.main.scene_brush_view.view.*
import java.io.File
import java.util.*
import java.util.concurrent.CopyOnWriteArrayList


class SceneEditActivity : AppCompatActivity(), ActorAdapter.OnActorDragListener, EditActorView.OnActorChangeListener,
        OnTouchBoMoSceneListener.OnSceneMoveListener, DialogInterface.OnDismissListener, FileUtils.OnSaveActorImage,
        OnTouchSceneListener.OnSceneTouchListener,LayerManagementDialog.OnLayerChange, EditTextDialog.OnSetDialogFinish,
        BrushView.OnSelectBrush {

    private lateinit var project: Project
    private var currentSceneIndex = 0
    private var currentLayerIndex = 6
    private var startRecordTime = 0L
    private var recordingTime = 0L
    private var swipeX = 0
    private var record = Record()
    private var resourceThemeBitmap = ArrayList<String>()
    private var resourceActorBitmap = ArrayList<ArrayList<String>>()
    private var resourceObjectPath = ArrayList<String>()
    private lateinit var sceneAdapter: SceneAdapter
    private var isRecord = false
    private var isAutoSwipe = false
    private var layerManagementDialog = LayerManagementDialog()
    private var editTextDialog = EditTextDialog()
    private val monitorSize = DisplayMetrics()
    private var timer = Timer()
    private val audioRecorder = AudioRecorder()
    companion object {
        const val PHOTO_FROM_GALLERY = 1
        const val LAYER_MANAGEMENT = "layer_management"
        const val ACTOR_HEIGHT = 400
        const val ACTOR_WIDTH = 400
        const val DIALOGUE_HEIGHT = 320
        const val DIALOGUE_WIDTH = 400
        const val ACTOR_INIT_POSITION_X = 200
        const val ACTOR_INIT_POSITION_Y = 200
        const val BACKGROUND_MOVE_RATE = 32
        const val BRUSH_SIZE = 16
        const val INIT_SCENE_NUMBER = 11
        const val TIME_STEP_FOR_FPS = 33L
        val listeners = CopyOnWriteArrayList<Runnable>()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_scene_edit)
        window.setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                WindowManager.LayoutParams.FLAG_FULLSCREEN)
        actionBar?.hide()
        supportActionBar?.hide()
        project = Gson().fromJson<Project>(intent.getStringExtra("project"), Project::class.java)
        resourceThemeBitmap = project.sceneResource as ArrayList<String>
        resourceActorBitmap = project.actorResource as ArrayList<ArrayList<String>>
        resourceObjectPath = ArrayList<String>().apply {
            project.themeAssetIndex.contains.obj.values.forEach {
                val path = "${project.assetFolder.path}/${it.smallFile}"
                add(path)
            }
        }
        windowManager.defaultDisplay.getMetrics(monitorSize)
        var layoutPrams = sceneEditView.layoutParams
        layoutPrams.width = monitorSize.widthPixels
        layoutPrams.height = monitorSize.widthPixels.div(2)
        sceneEditView.layoutParams = layoutPrams
        project.story?.let {
            it.scenes?.let{
                it[0].sceneWidth = monitorSize.widthPixels
                for (i in 0 until INIT_SCENE_NUMBER){
                    addScene(0)
                }
                sceneEditView.scene = it[0]
            }
        }
        ic_scene_edit_activity_back.setOnClickListener {
            val pd = ProgressDialog(this).apply {
                setCancelable(false)
            }
            FileUtils.saveProject(this, project, 1920,1080)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .doOnSubscribe { pd.show() }
                    .doFinally {
                        pd.dismiss()
                        this.onBackPressed()}
                    .subscribe {
                        listeners.forEach {it.run()}
                    }
        }
        initControlBarView()
        initNavigationFunction()
        initRecord()
    }

    override fun onResume() {
        super.onResume()
        window.decorView.apply {
            systemUiVisibility = (View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_LOW_PROFILE
                    or View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)
        }
    }

    private fun initControlBarView() {
        controlBarView.setBtnSelected(6)
        switchToPreViewLayer()
        project.story?.let {
            it.scenes?.let {
                if (it[currentSceneIndex].record != null) {
                    controlBarView.visibility = View.INVISIBLE
                    sceneEditView.setOnTouchListener(null)
                } else {
                    controlBarView.visibility = View.VISIBLE
                }
            }
        }
        controlBarView.setonClickSwipeLayerBtn(View.OnClickListener {
            switchToPreViewLayer()
        })
        controlBarView.setonClickLayerSceneBtn(View.OnClickListener {
            controlBarView.setBtnSelected(5)
            layerManagementDialog = LayerManagementDialog()
            project.story?.let {
                it.scenes?.let{
                    layerManagementDialog.setData(it[currentSceneIndex])
                }
            }
            layerManagementDialog.setListener(this)
            layerManagementDialog.show(supportFragmentManager,LAYER_MANAGEMENT)
        })
        controlBarView.setonClickLayerBackgroundBtn(View.OnClickListener {
            switchLayer(4)
        })
        controlBarView.setonClickLayer4Btn(View.OnClickListener {
            switchLayer(3)
        })
        controlBarView.setonClickLayer3Btn(View.OnClickListener {
            switchLayer(2)
        })
        controlBarView.setonClickLayer2Btn(View.OnClickListener {
            switchLayer(1)
        })
        controlBarView.setonClickLayer1Btn(View.OnClickListener {
            switchLayer(0)
        })
    }

    private fun initNavigationBarView(position: Int) {
        project.story?.let {
            it.scenes?.let {
                if (position != currentSceneIndex) {
                    initNavigationFunction()
                } else {
                    navigationBar.removeAllViews()
                    var editSceneView = EditSceneView(this)
                    navigationBar.addView(editSceneView)
                    setControlSceneView()
                    editSceneView.setOnClickDoneListener(View.OnClickListener {
                        initNavigationFunction()
                    })
                    editSceneView.setOnClickStartOverListener(View.OnClickListener {
                        project.story?.let {
                            it.scenes?.let{
                                it[currentSceneIndex].record = null
                                sceneEditView.scene = it[currentSceneIndex]
                                initNavigationBarView(currentSceneIndex)
                            }
                        }
                    })
                    editSceneView.setOnClickReplaceListener(View.OnClickListener {
                        viewContainer.removeAllViews()
                        var addSceneView = AddSceneView(this)
                        viewContainer.addView(addSceneView)
                        val layoutManager = LinearLayoutManager(this)
                        layoutManager.orientation = LinearLayoutManager.HORIZONTAL
                        var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
                        for (i in 0 until resourceThemeBitmap.size) {
                            onSceneSelectedArrayList.add(replaceScene(i))
                        }
                        var addSceneAdapter = AddSceneAdapter(this, resourceThemeBitmap, onSceneSelectedArrayList)
                        var newSceneRecyclerView = addSceneView.findViewById<RecyclerView>(R.id.new_scene_recycler_view)
                        newSceneRecyclerView.layoutManager = layoutManager
                        newSceneRecyclerView.adapter = addSceneAdapter
                    })
                    editSceneView.setOnClickDeleteListener(View.OnClickListener {
                        project.story?.let {
                            it.scenes?.let{
                                if (it.size <= 1) { return@let }
                                it.removeAt(currentSceneIndex)
                                when(currentSceneIndex >= it.size){
                                    true -> {
                                        currentSceneIndex = 0
                                        sceneEditView.scene = it[0]
                                    }
                                    false -> sceneEditView.scene = it[currentSceneIndex]
                                }
                                initNavigationBarView(currentSceneIndex)
                                setControlSceneView()
                            }
                        }
                    })
                }
            }
        }
    }

    private fun initNavigationFunction(){
        var navigationBarView = NavigationBarView(this)
        navigationBar.addView(navigationBarView)
        navigationBarView.setonClickSceneBtn(View.OnClickListener {
            initControlSceneView()
        })
        navigationBarView.setonClickActorBtn(View.OnClickListener {
            initActorRecyclerView()
        })
        navigationBarView.setonClickBrushBtn(View.OnClickListener {
            initBrushView()
        })
        navigationBarView.setonClickDialogueBtn(View.OnClickListener {
            initDialogueView()
        })
        navigationBarView.setonClickImageBtn(View.OnClickListener {
            pickFromGallery()
        })
        navigationBarView.setonClickSettingBtn(View.OnClickListener {
            var popupWindow = PopupWindow(this)
            popupWindow.contentView = createSettingPopupView(popupWindow)
            popupWindow.width = ViewGroup.LayoutParams.WRAP_CONTENT
            popupWindow.height = ViewGroup.LayoutParams.WRAP_CONTENT
            popupWindow.isOutsideTouchable = true
            var valueXInPixels = resources.getDimension(R.dimen.popup_view_setting_height) + resources.getDimension(R.dimen.navigation_bar_height)
            popupWindow.showAsDropDown(navigationBarView.settings, 0, -(valueXInPixels).toInt(), Gravity.CENTER_HORIZONTAL)

        })
    }

    private fun createSettingPopupView(popupWindow: PopupWindow): View{
        val popupView = LayoutInflater.from(this).inflate(R.layout.popupview_setting, null)
        when (isAutoSwipe) {
            true -> {
                popupView.rb_manual.isChecked  = false
                popupView.rb_auto_swipe.isChecked = true
            }
            false -> {
                popupView.rb_manual.isChecked  = true
                popupView.rb_auto_swipe.isChecked = false
            }
        }

        popupView.rg_settings.setOnCheckedChangeListener(object : RadioGroup.OnCheckedChangeListener{
            override fun onCheckedChanged(group: RadioGroup?, checkedId: Int) {
                when (checkedId) {
                    popupView.rb_manual.id -> {
                        isAutoSwipe = false
                    }
                    popupView.rb_auto_swipe.id ->{
                        isAutoSwipe = true
                    }
                }
            }
        })
        return popupView
    }
    private fun initBrushView(){
        if (navigationBar.getChildAt(0) is BrushView) {
            navigationBar.removeAllViews()
            startRecord.visibility = View.VISIBLE
            return
        }
        if (currentLayerIndex == 6){
            switchLayer(0)
            initBrushView()
        }
        startRecord.visibility = View.INVISIBLE
        viewContainer.removeAllViews()
        navigationBar.removeAllViews()
        var brushView = BrushView(this)
        brushView.setColorListener(this)
        navigationBar.addView(brushView)
        brushView.gravity = Gravity.CENTER_HORIZONTAL
        brushView.layoutParams.width = monitorSize.widthPixels
        var initBrush = Brush().apply {
            brushSize = BRUSH_SIZE
            brushColor = DialogueColorData.colorMap[1]
            brushType = 0
        }
        brushView.setBrush(initBrush, object : SeekBar.OnSeekBarChangeListener{
            override fun onProgressChanged(p0: SeekBar?, p1: Int, p2: Boolean) {
                brushView.tv_brush_view_size.text = "Size: " + p1.toString() + "px"
                var brush = paint_view.brush
                brush.brushSize = p1
                paint_view.brush = brush
            }
            override fun onStartTrackingTouch(p0: SeekBar?) {
            }
            override fun onStopTrackingTouch(p0: SeekBar?) {
            }
        })
        brushView.setOnDoneClick(View.OnClickListener {
            navigationBar.removeAllViews()
            paint_view.visibility = View.INVISIBLE
            startRecord.visibility = View.VISIBLE
            initNavigationBarView(currentSceneIndex)
            FileUtils.saveBitmapForActor(this, paint_view.bitmap, System.currentTimeMillis().toString())
            project.story?.let {
                it.scenes?.let{
                    sceneEditView.scene = it[currentSceneIndex]
                }
            }
        })
        paint_view.init()
        paint_view.layoutParams.width = monitorSize.widthPixels
        paint_view.layoutParams.height = monitorSize.widthPixels.div(2)
        paint_view.bringToFront()
        paint_view.visibility = View.VISIBLE
        paint_view.setOnTouchListener(paint_view)
        paint_view.brush = initBrush
    }

    private fun initActorRecyclerView() {
        if (viewContainer.getChildAt(0) is SelectActorView || viewContainer.getChildAt(0) is ObjectView) {
            viewContainer.removeAllViews()
            startRecord.visibility = View.VISIBLE
            return
        } else {
            viewContainer.removeAllViews()
        }
        startRecord.visibility = View.INVISIBLE
        if (currentLayerIndex == 6){
            switchLayer(0)
            initActorRecyclerView()
        }
        var selectActorView = SelectActorView(this)
        val layoutManager = LinearLayoutManager(this)
        layoutManager.orientation = LinearLayoutManager.HORIZONTAL
        var actorAdapter = ActorAdapter(this, resourceActorBitmap, sceneEditView.findViewById(currentLayerIndex), this)
        var actorRecyclerView = selectActorView.findViewById<RecyclerView>(R.id.select_actor_recycler_view)
        actorRecyclerView.layoutManager = layoutManager
        actorRecyclerView.adapter = actorAdapter
        viewContainer.addView(selectActorView)
        selectActorView.layoutParams.width = monitorSize.widthPixels
        selectActorView.btn_object.setOnClickListener {
            viewContainer.removeAllViews()
            var objectView = ObjectView(this)
            objectView.setData(resourceObjectPath)
            sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(objectDragListener())
            viewContainer.addView(objectView)
            objectView.layoutParams.width = monitorSize.widthPixels
            objectView.btn_character.setOnClickListener {
                viewContainer.removeAllViews()
                initActorRecyclerView()
            }
        }
    }
    private fun initDialogueView(){
        if (viewContainer.getChildAt(0) is DialogueView) {
            viewContainer.removeAllViews()
            startRecord.visibility = View.VISIBLE
            return
        } else {
            viewContainer.removeAllViews()
        }
        startRecord.visibility = View.INVISIBLE
        if (currentLayerIndex == 6){
            switchLayer(0)
            initDialogueView()
            return
        }
        var dialogueView = DialogueView(this)
        sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(dragListener())
        viewContainer.addView(dialogueView)
        dialogueView.layoutParams.width = monitorSize.widthPixels
    }


    private fun initControlSceneView() {
        if (viewContainer.getChildAt(0) is ControlSceneView) {
            viewContainer.removeAllViews()
            return
        }
        setControlSceneView()
    }

    private fun setControlSceneView(){
        var projectBitmaps = ArrayList<Bitmap?>()
        var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
        var recordTimeArrayList = ArrayList<Long?>()
        project.story?.let {
            it.scenes?.let{
                for (i in 0 until it.size) {
                    onSceneSelectedArrayList.add(switchScene(i))
                    projectBitmaps.add(SceneDrawer.drawScene(this,it[i],0,200,100))
                    recordTimeArrayList.add(it[i].record?.period)
                }
            }
        }
        sceneAdapter = SceneAdapter(this, projectBitmaps, onSceneSelectedArrayList, currentSceneIndex, recordTimeArrayList)
        var controlSceneView = ControlSceneView(this)
        controlSceneView.setonClickAddBtn(View.OnClickListener {
            viewContainer.removeAllViews()
            var addSceneView = AddSceneView(this)
            viewContainer.addView(addSceneView)
            val layoutManager = LinearLayoutManager(this)
            layoutManager.orientation = LinearLayoutManager.HORIZONTAL
            var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
            for (i in 0 until resourceThemeBitmap.size) {
                onSceneSelectedArrayList.add(onClickAddSceneListener(i))
            }
            var addSceneAdapter = AddSceneAdapter(this, resourceThemeBitmap, onSceneSelectedArrayList)
            var newSceneRecyclerView = addSceneView.findViewById<RecyclerView>(R.id.new_scene_recycler_view)
            newSceneRecyclerView.layoutManager = layoutManager
            newSceneRecyclerView.adapter = addSceneAdapter
        })
        controlSceneView.setonClickDuplicateBtn(View.OnClickListener {
            project.story?.let {
                it.scenes?.let {
                    it.add(currentSceneIndex + 1, it[currentSceneIndex])
                }
            }
            var sceneAdapter = controlSceneView.findViewById<RecyclerView>(R.id.scene_recycler_view).adapter as SceneAdapter
            var projectBitmaps = ArrayList<Bitmap?>()
            var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
            project.story?.let {
                it.scenes?.let{
                    for (i in 0 until it.size) {
                        onSceneSelectedArrayList.add(switchScene(i))
                        projectBitmaps.add(SceneDrawer.drawScene(this,it[i],0,200,100))
                    }
                }
            }
            sceneAdapter.update(projectBitmaps, onSceneSelectedArrayList)
        })
        val layoutManager = LinearLayoutManager(this)
        layoutManager.orientation = LinearLayoutManager.HORIZONTAL
        var sceneRecyclerView = controlSceneView.findViewById<RecyclerView>(R.id.scene_recycler_view)
        sceneRecyclerView.layoutManager = layoutManager
        sceneRecyclerView.adapter = sceneAdapter
        viewContainer.removeAllViews()
        viewContainer.addView(controlSceneView)
        startRecord.visibility = View.VISIBLE
    }

    override fun onPause() {
        super.onPause()
        if (isRecord) {
            cancelRecording()
        }
    }

    private fun cancelRecording() {
        audioRecorder.cancelRecording()

        tv_scene_activity_record_time.visibility = View.INVISIBLE
        timer.cancel()
        timer = Timer()
        recordingTime = 0L
        tv_scene_activity_record_time.text = "Recording 00:00"
        isRecord = false
        startRecord.setImageDrawable(resources.getDrawable(R.drawable.ic_btn_record))
        navigationBar.visibility = View.VISIBLE
        initControlBarView()
        initControlSceneView()
        initRecordFinishView()
    }

    private fun getRecordFilePath(): String {
        val projectFolder = File(Config.PROJECTS_FOLDER, project.name)
        return "${projectFolder.absolutePath}/Record/${System.currentTimeMillis()}.mp3"
    }

    private fun initRecord(){
        var recordPath: String? = null
        startRecord.setOnClickListener(View.OnClickListener {
            if (!isRecord) {
                //  delete record
                project.story?.let {
                    it.scenes?.let{
                        it[currentSceneIndex].record = null
                        if (it[currentSceneIndex].recordPath != null)
                            File(it[currentSceneIndex].recordPath).delete()
                    }
                }
                recordPath = getRecordFilePath()
                audioRecorder.prepare(File(recordPath))
                audioRecorder.startRecording()
                startRecord.setImageDrawable(resources.getDrawable(R.drawable.ic_btn_record_stop))
                startRecordTime = System.currentTimeMillis()
                isRecord = true
                switchToPreViewLayer()
                navigationBar.visibility = View.INVISIBLE
                tv_scene_activity_record_time.visibility = View.VISIBLE
                timer.schedule(object:TimerTask(){
                    override fun run() {
                        runOnUiThread {
                            recordingTime ++
                            tv_scene_activity_record_time.text = "Recording " + "0" + recordingTime.div(60).toString() + ":" +
                                    recordingTime.rem(60).div(10).toString() + recordingTime.rem(60).rem(10).toString()
                        }
                    }
                }, 1000, 1000)
            } else {
                audioRecorder.stopRecording()
                tv_scene_activity_record_time.visibility = View.INVISIBLE
                timer.cancel()
                timer = Timer()
                recordingTime = 0L
                tv_scene_activity_record_time.text = "Recording 00:00"
                isRecord = false
                record.period = System.currentTimeMillis() - startRecordTime
                if (!isAutoSwipe){
                    project.story?.let {
                        it.scenes?.let{
                            it[currentSceneIndex].record = record
                            it[currentSceneIndex].recordPath = recordPath
                        }
                    }
                } else {
                    var autoSwipeRecord = Record()
                    autoSwipeRecord.period = record.period
                    for (i in 0 until record.period step TIME_STEP_FOR_FPS)
                    autoSwipeRecord.tracks.apply {
                        var track = Track().apply {
                            when (i <= record.period.div(2)){
                                true -> {
                                    positionX = (-monitorSize.widthPixels.div(2)) + i.div(TIME_STEP_FOR_FPS).toInt() * monitorSize.widthPixels.div(record.period.div(2 * TIME_STEP_FOR_FPS).toInt())
                                    time = i
                                }
                                false -> {
                                    var j = i - record.period.div(2)
                                    positionX = (monitorSize.widthPixels.div(2)) - j.div(TIME_STEP_FOR_FPS).toInt() * monitorSize.widthPixels.div(record.period.div(2 * TIME_STEP_FOR_FPS).toInt())
                                    time = i
                                }
                            }
                        }
                        add(track)
                    }
                    project.story?.let {
                        it.scenes?.let{
                            it[currentSceneIndex].record = autoSwipeRecord
                            it[currentSceneIndex].recordPath = recordPath
                        }
                    }
                }

                startRecord.setImageDrawable(resources.getDrawable(R.drawable.ic_btn_record))
                navigationBar.visibility = View.VISIBLE
                initControlBarView()
                initControlSceneView()
                initRecordFinishView()
            }
        })
    }

    private fun initRecordFinishView(){
        navigationBar.removeAllViews()
        var recordFinishView = RecordFinishView(this)
        recordFinishView.setOnNextSceneClickListener(View.OnClickListener {
            currentSceneIndex++
            project.story?.let {
                it.scenes?.let{
                    when(currentSceneIndex < it.size){
                        true -> {
                            switchToPreViewLayer()
                            sceneEditView.scene = it[currentSceneIndex]
                        }
                        false -> {
                            currentSceneIndex = 0
                            sceneEditView.scene = it[currentSceneIndex]
                        }
                    }
                }
            }
            initControlBarView()
            initNavigationBarView(currentSceneIndex)
        })
        recordFinishView.setOnFinishClickListener(View.OnClickListener {
            val pd = ProgressDialog(this).apply {
                         setCancelable(false)
                     }
            FileUtils.saveProject(this, project, 1920,1080)
                     .subscribeOn(Schedulers.io())
                     .observeOn(AndroidSchedulers.mainThread())
                     .doOnSubscribe { pd.show() }
                     .doFinally { pd.dismiss() }
                     .subscribe {
                         listeners.forEach {it.run()}
                     }
            val intent = Intent(this, MovieEditActivity::class.java)
            intent.putExtra(MovieEditActivity.PROJECT_KEY, Gson().toJson(project))
            startActivity(intent)
        })
        navigationBar.addView(recordFinishView)
    }
    private fun switchScene(position: Int): View.OnClickListener {
        return View.OnClickListener {
            var oldSceneIndex = currentSceneIndex
            currentSceneIndex = position
            switchToPreViewLayer()
            project.story?.let {
                it.scenes?.let{
                    sceneEditView.scene = it[position]
                }
            }
            record = Record()
            initControlBarView()
            initNavigationBarView(oldSceneIndex)
        }
    }

    private fun switchLayer(layerIndex: Int){
        viewContainer.removeAllViews()
        if(currentLayerIndex != 6) {
            project.story?.let {
                it.scenes?.let {
                    for (i in 0 until it[currentSceneIndex].layers[currentLayerIndex].actors.size) {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect = false
                    }
                }
            }
        }
        currentLayerIndex = layerIndex
        controlBarView.setBtnSelected(4 - layerIndex)
        project.story?.let {
            it.scenes?.let {
                for (layer in it[currentSceneIndex].layers) {
                    when(it[currentSceneIndex].layers.indexOf(layer)) {
                        currentLayerIndex ->
                            for (actor in it[currentSceneIndex].layers[currentLayerIndex].actors) {
                                actor.isMovable = true
                            }
                        else ->
                            for (actor in it[currentSceneIndex].layers[it[currentSceneIndex].layers.indexOf(layer)].actors) {
                                actor.isMovable = false
                            }
                    }
                }

                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        val sceneListener = OnTouchSceneListener()
        sceneListener.setCallBack(this)
        sceneEditView.setOnTouchListener(sceneListener)
        sceneEditView.x = 0
        setActorDropLayer()
        sceneEditView.setLayerVisible(layerIndex)
    }

    private fun switchToPreViewLayer(){
        viewContainer.removeAllViews()
        currentLayerIndex = 6
        controlBarView.setBtnSelected(currentLayerIndex)
        project.story?.let {
            it.scenes?.let {
                for (layer in it[currentSceneIndex].layers) {
                    for (actor in layer.actors) {
                        actor.isMovable = false
                        actor.isSelect = false
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        val boMoSceneListener = OnTouchBoMoSceneListener()
        boMoSceneListener.setCallBack(this)
        sceneEditView.setOnTouchListener(boMoSceneListener)
        sceneEditView.x = 0
        if (viewContainer.getChildAt(0) is SelectActorView) {
            viewContainer.removeAllViews()
        }
    }


    private fun setActorDropLayer() {
        if (viewContainer.getChildAt(0) is SelectActorView) {
            var selectActorView = SelectActorView(this)
            val layoutManager = LinearLayoutManager(this)
            layoutManager.orientation = LinearLayoutManager.HORIZONTAL
            var actorAdapter = ActorAdapter(this, resourceActorBitmap, sceneEditView.findViewById(currentLayerIndex), this)
            var actorRecyclerView = selectActorView.findViewById<RecyclerView>(R.id.select_actor_recycler_view)
            actorRecyclerView.layoutManager = layoutManager
            actorRecyclerView.adapter = actorAdapter
            selectActorView.btn_object.setOnClickListener {
                viewContainer.removeAllViews()
                var objectView = ObjectView(this)
                objectView.setData(resourceObjectPath)
                sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(objectDragListener())
                viewContainer.addView(objectView)
                objectView.layoutParams.width = monitorSize.widthPixels
                objectView.btn_character.setOnClickListener {
                    viewContainer.removeAllViews()
                    initActorRecyclerView()
                }
            }
            viewContainer.addView(selectActorView)
        }
    }

    private fun onClickAddSceneListener(position: Int): View.OnClickListener {
        return View.OnClickListener {
            addScene(position)
            initControlSceneView()
            initNavigationBarView(currentSceneIndex)
        }
    }

    private fun addScene(position: Int){
        var scene = Scene()
        for (i in 5 until 0){
            val layer = Layer()
            layer.id = i
            scene.layers.add(layer)
        }
        scene.backgroundPath = resourceThemeBitmap[position]
        project.story?.let {
            it.scenes?.let{
                scene.sceneWidth = monitorSize.widthPixels
                it.add(scene)
            }
        }
    }

    private fun replaceScene(position: Int): View.OnClickListener {
        return View.OnClickListener {
            project.story?.let {
                it.scenes?.let{
                    it[currentSceneIndex].backgroundPath = resourceThemeBitmap[position]
                    sceneEditView.scene = it[currentSceneIndex]
                }
            }
            initControlSceneView()
            initNavigationBarView(currentSceneIndex)
        }
    }
    private fun setActorPositionZ(actors: ArrayList<Actor>){
        for (actor in actors) {
            actor.positionZ = actors.indexOf(actor)
        }
    }

    private fun pickFromGallery() {
        val intent = Intent()
        intent.type = "image/*"
        intent.action = Intent.ACTION_GET_CONTENT
        startActivityForResult(intent, PHOTO_FROM_GALLERY)
    }

    private fun addActor(positionX: Int, positionY: Int, resourcePath: String) {
        project.story?.let {
            it.scenes?.let{
                val actorData = Actor()
                actorData.positionX = positionX
                actorData.positionY = positionY
                actorData.positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                actorData.sideLength = ACTOR_WIDTH
                actorData.sideHeight = ACTOR_HEIGHT
                actorData.resourcePath = resourcePath
                actorData.parentLayerIndex = currentLayerIndex
                actorData.isMovable = true
                it[currentSceneIndex].layers[currentLayerIndex].actors.add(actorData)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        setActorDropLayer()
        sceneEditView.setLayerVisible(currentLayerIndex)
    }

    private fun addObject(positionX: Int, positionY: Int, resourcePath: String) {
        project.story?.let {
            it.scenes?.let{
                val actorData = Actor()
                actorData.positionX = positionX
                actorData.positionY = positionY
                actorData.positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                actorData.sideLength = ACTOR_WIDTH
                actorData.sideHeight = ACTOR_HEIGHT
                actorData.resourcePath = resourcePath
                actorData.parentLayerIndex = currentLayerIndex
                actorData.isMovable = true
                it[currentSceneIndex].layers[currentLayerIndex].actors.add(actorData)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(objectDragListener())
    }
    private fun addDialogue(positionX: Int, positionY: Int, resource: Int) {
        project.story?.let {
            it.scenes?.let{
                val dialog = Actor()
                dialog.resourcePath = resource.toString()
                dialog.positionX = positionX
                dialog.positionY = positionY
                dialog.positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                dialog.sideLength = DIALOGUE_WIDTH
                dialog.sideHeight = DIALOGUE_HEIGHT
                dialog.parentLayerIndex = currentLayerIndex
                dialog.isMovable = true
                dialog.isDialogue = true
                dialog.dialogType = DialogueColorData.getDialogColorType(resource).first
                dialog.dialogColor = DialogueColorData.getDialogColorType(resource).second
                it[currentSceneIndex].layers[currentLayerIndex].actors.add(dialog)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(dragListener())
    }
    override fun onActorDropFinish(positionX: Int, positionY: Int, resourcePath: String) {
        addActor(positionX, positionY, resourcePath)
    }

    override fun onActorChange(actor: Actor) {
        project.story?.let {
            it.scenes?.let {
                it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ] = actor
            }
        }
        sceneEditView.setLayerVisible(currentLayerIndex)
    }

    override fun onActorSelected(actor: Actor) {
        project.story?.let {
            it.scenes?.let {
                for(i in 0 until it[currentSceneIndex].layers[currentLayerIndex].actors.size){
                    if(actor.positionZ == i) {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect =
                                !it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect
                    } else {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect = false
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        sceneEditView.setLayerVisible(currentLayerIndex)
    }

    override fun onActorLongClick(actor: Actor) {
        project.story?.let {
            it.scenes?.let {
                for(i in 0 until it[currentSceneIndex].layers[currentLayerIndex].actors.size){
                    if(actor.positionZ == i) {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect =
                                !it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect
                    } else {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect = false
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        sceneEditView.setLayerVisible(currentLayerIndex)
        var popupWindow = PopupWindow(this)
        popupWindow.contentView = when(actor.isDialogue) { true -> createDialogPopupView(actor, popupWindow)
                                                           false -> createActorPopupView(actor, popupWindow)}
        popupWindow.width = ViewGroup.LayoutParams.WRAP_CONTENT
        popupWindow.height = ViewGroup.LayoutParams.WRAP_CONTENT
        popupWindow.isOutsideTouchable = true
        popupWindow.showAtLocation(sceneEditView, Gravity.START,actor.positionX,actor.positionY)
    }

    private fun createActorPopupView(actor: Actor, popupWindow: PopupWindow): View{
        val popupView = LayoutInflater.from(this).inflate(R.layout.control_actor_dialog, null)
        popupView.duplicate.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    val actorData = Actor().apply {
                        positionX = actor.positionX + 100
                        positionY = actor.positionY + 100
                        positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                        sideLength = actor.sideLength
                        sideHeight = actor.sideHeight
                        resourcePath = actor.resourcePath
                        parentLayerIndex = currentLayerIndex
                        isMovable = true
                    }
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(actorData)
                    setActorPositionZ(it[currentSceneIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.opacity.setOnClickListener {
            popupWindow.dismiss()
            var popupWindow = PopupWindow(this)
            var popupView = LayoutInflater.from(this).inflate(R.layout.popupview_opacity_dialog, null)
            popupWindow.contentView = popupView
            popupWindow.width = ViewGroup.LayoutParams.WRAP_CONTENT
            popupWindow.height = ViewGroup.LayoutParams.WRAP_CONTENT
            popupWindow.isOutsideTouchable = false
            popupWindow.showAtLocation(sceneEditView, Gravity.START,actor.positionX,0)
            popupView.sb_opacity_dialog_opacity.progress = (actor.opacity * 100).toInt()
            popupView.tv_opacity_dialog_opacity.text = "Opacity : " + (actor.opacity * 100).toInt().toString() + "%"
            popupView.sb_opacity_dialog_opacity.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener{
                override fun onProgressChanged(p0: SeekBar?, p1: Int, p2: Boolean) {
                    popupView.tv_opacity_dialog_opacity.text = "Opacity : " + p1.toString() + "%"
                    project.story?.let {
                        it.scenes?.let {
                            it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].opacity = p1.toFloat().div(100)
                            sceneEditView.scene = it[currentSceneIndex]
                        }
                    }
                }
                override fun onStartTrackingTouch(p0: SeekBar?) {
                }
                override fun onStopTrackingTouch(p0: SeekBar?) {
                }
            })
            popupView.tv_opacity_dialog_ok.setOnClickListener {
                popupWindow.dismiss()
            }
        }
        popupView.Mirror.setOnClickListener {
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].isMirror =
                            !it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].isMirror
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.bring_to_front.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.remove(actor)
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(actor)
                    setActorPositionZ(it[currentLayerIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.send_to_back.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.remove(actor)
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(0, actor)
                    setActorPositionZ(it[currentLayerIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.delete.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.removeAt(actor.positionZ)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        return popupView
    }

    private fun createDialogPopupView(actor: Actor, popupWindow: PopupWindow): View{
        val popupView = LayoutInflater.from(this).inflate(R.layout.view_control_dialogue_dialog, null)
        popupView.edit_content_dialogue.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    editTextDialog = EditTextDialog()
                    editTextDialog.setCallBack(this)
                    editTextDialog.setDialogue(actor)
                    editTextDialog.show(supportFragmentManager,LAYER_MANAGEMENT)
                    popupWindow.dismiss()
                }
            }
        }
        popupView.duplicate_dialogue.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    val actorData = Actor().apply {
                        positionX = actor.positionX + 100
                        positionY = actor.positionY + 100
                        positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                        sideLength = actor.sideLength
                        sideHeight = actor.sideHeight
                        resourcePath = actor.resourcePath
                        parentLayerIndex = currentLayerIndex
                        isMovable = true
                        text = actor.text
                        textColor = actor.textColor
                        textAlign =  actor.textAlign
                        isDialogue = actor.isDialogue
                        opacity = actor.opacity
                    }
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(actorData)
                    setActorPositionZ(it[currentSceneIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.Mirror_dialogue.setOnClickListener {
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].isMirror =
                            !it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].isMirror
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.layout_dialogue_dialog_color.setOnClickListener {
            popupWindow.dismiss()
            var popupWindow = PopupWindow(this)
            var popupView = LayoutInflater.from(this).inflate(R.layout.popupview_color_dialog, null)
            when (actor.dialogColor){
                0 -> popupView.iv_dialogue_set_color_white.setImageResource(R.drawable.ic_done)
                1 -> popupView.iv_dialogue_set_color_yellow.setImageResource(R.drawable.ic_done)
                2 -> popupView.iv_dialogue_set_color_blue.setImageResource(R.drawable.ic_done)
                3 -> popupView.iv_dialogue_set_color_green.setImageResource(R.drawable.ic_done)
                4 -> popupView.iv_dialogue_set_color_orange.setImageResource(R.drawable.ic_done)
            }
            popupView.iv_dialogue_set_color_white.setOnClickListener {
                popupView.iv_dialogue_set_color_white.setImageResource(R.drawable.ic_done)
                popupView.iv_dialogue_set_color_blue.setImageDrawable(null)
                popupView.iv_dialogue_set_color_yellow.setImageDrawable(null)
                popupView.iv_dialogue_set_color_green.setImageDrawable(null)
                popupView.iv_dialogue_set_color_orange.setImageDrawable(null)
                project.story?.let {
                    it.scenes?.let {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].resourcePath = DialogueColorData.dialogMap[Pair(actor.dialogType,0)].toString()
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].dialogColor = 0
                        sceneEditView.scene = it[currentSceneIndex]
                    }
                }
            }
            popupView.iv_dialogue_set_color_blue.setOnClickListener {
                popupView.iv_dialogue_set_color_white.setImageDrawable(null)
                popupView.iv_dialogue_set_color_blue.setImageResource(R.drawable.ic_done)
                popupView.iv_dialogue_set_color_yellow.setImageDrawable(null)
                popupView.iv_dialogue_set_color_green.setImageDrawable(null)
                popupView.iv_dialogue_set_color_orange.setImageDrawable(null)
                project.story?.let {
                    it.scenes?.let {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].resourcePath = DialogueColorData.dialogMap[Pair(actor.dialogType,2)].toString()
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].dialogColor = 2
                        sceneEditView.scene = it[currentSceneIndex]
                    }
                }
            }
            popupView.iv_dialogue_set_color_yellow.setOnClickListener {
                popupView.iv_dialogue_set_color_white.setImageDrawable(null)
                popupView.iv_dialogue_set_color_blue.setImageDrawable(null)
                popupView.iv_dialogue_set_color_yellow.setImageResource(R.drawable.ic_done)
                popupView.iv_dialogue_set_color_green.setImageDrawable(null)
                popupView.iv_dialogue_set_color_orange.setImageDrawable(null)
                project.story?.let {
                    it.scenes?.let {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].resourcePath = DialogueColorData.dialogMap[Pair(actor.dialogType,1)].toString()
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].dialogColor = 1
                        sceneEditView.scene = it[currentSceneIndex]
                    }
                }
            }
            popupView.iv_dialogue_set_color_green.setOnClickListener {
                popupView.iv_dialogue_set_color_white.setImageDrawable(null)
                popupView.iv_dialogue_set_color_blue.setImageDrawable(null)
                popupView.iv_dialogue_set_color_yellow.setImageDrawable(null)
                popupView.iv_dialogue_set_color_green.setImageResource(R.drawable.ic_done)
                popupView.iv_dialogue_set_color_orange.setImageDrawable(null)
                project.story?.let {
                    it.scenes?.let {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].resourcePath = DialogueColorData.dialogMap[Pair(actor.dialogType,3)].toString()
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].dialogColor = 3
                        sceneEditView.scene = it[currentSceneIndex]
                    }
                }
            }
            popupView.iv_dialogue_set_color_orange.setOnClickListener {
                popupView.iv_dialogue_set_color_white.setImageDrawable(null)
                popupView.iv_dialogue_set_color_blue.setImageDrawable(null)
                popupView.iv_dialogue_set_color_yellow.setImageDrawable(null)
                popupView.iv_dialogue_set_color_green.setImageDrawable(null)
                popupView.iv_dialogue_set_color_orange.setImageResource(R.drawable.ic_done)
                project.story?.let {
                    it.scenes?.let {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].resourcePath = DialogueColorData.dialogMap[Pair(actor.dialogType,4)].toString()
                        it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ].dialogColor = 4
                        sceneEditView.scene = it[currentSceneIndex]
                    }
                }
            }
            popupView.tv_color_dialog_ok.setOnClickListener {
                popupWindow.dismiss()
            }
            popupWindow.width = ViewGroup.LayoutParams.WRAP_CONTENT
            popupWindow.height = ViewGroup.LayoutParams.WRAP_CONTENT
            popupWindow.contentView = popupView
            popupWindow.isOutsideTouchable = false
            popupWindow.showAtLocation(sceneEditView, Gravity.START,actor.positionX,0)
        }
        popupView.bring_to_front_dialogue.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.remove(actor)
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(actor)
                    setActorPositionZ(it[currentLayerIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.send_to_back_dialogue.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.remove(actor)
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(0, actor)
                    setActorPositionZ(it[currentLayerIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.delete_dialogue.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.removeAt(actor.positionZ)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        return popupView
    }
    override fun onSceneMove(dX: Int) {
        if ( (BACKGROUND_MOVE_RATE * monitorSize.widthPixels.div(BACKGROUND_MOVE_RATE * 2)) >= (swipeX + dX) && (swipeX + dX) > (-(BACKGROUND_MOVE_RATE * monitorSize.widthPixels.div(BACKGROUND_MOVE_RATE * 2)))) {
            sceneEditView.x = swipeX + dX
            if (isRecord) {
                record?.tracks.let {
                    val track = Track()
                    track.positionX = swipeX + dX
                    track.time = System.currentTimeMillis() - startRecordTime
                    record.tracks.add(track)
                }
            }
        }
    }

    override fun onSceneReadyToMove() {
        swipeX = sceneEditView.x
    }

    override fun onSceneFinishMove() {
        swipeX = 0
    }

    override fun onDismiss(dialog: DialogInterface?) {
        project.story?.let {
            it.scenes?.let {
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        window.decorView.apply {
            systemUiVisibility = (View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_LOW_PROFILE
                    or View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)
        }
        initControlBarView()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            PHOTO_FROM_GALLERY ->
                when (resultCode) {
                    Activity.RESULT_OK -> if (data != null) {
                        FileUtils.saveImageForActor(this, data.data, System.currentTimeMillis().toString())
                    }
                    Activity.RESULT_CANCELED -> {
                    }
                }
        }
    }

    override fun onSaveActorImage(filePath: String) {
        addActor(ACTOR_INIT_POSITION_X, ACTOR_INIT_POSITION_Y, filePath)
    }

    override fun onLayerChange(layerFrom: Int, layerTo: Int, position: Int){
        project.story?.let {
            it.scenes?.let {
                var actor = it[currentSceneIndex].layers[layerFrom - 1].actors[position]
                it[currentSceneIndex].layers[layerTo - 1].actors.add(actor)
                it[currentSceneIndex].layers[layerFrom - 1].actors.removeAt(position)
                layerManagementDialog.setData(it[currentSceneIndex])
                setActorPositionZ(it[currentSceneIndex].layers[layerFrom - 1].actors)
                setActorPositionZ(it[currentSceneIndex].layers[layerTo - 1].actors)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
    }

    private fun dragListener() : View.OnDragListener {
        return View.OnDragListener {v, event ->
            when (event.action) {
                DragEvent.ACTION_DRAG_STARTED -> {
                    return@OnDragListener true
                }
                DragEvent.ACTION_DROP -> {
                    addDialogue(event.x.toInt() - 100, event.y.toInt() - 100, event.clipData.getItemAt(0).text.toString().toInt())
                    return@OnDragListener true
                }
            }
            return@OnDragListener true
        }
    }

    private fun objectDragListener() : View.OnDragListener {
        return View.OnDragListener {v, event ->
            when (event.action) {
                DragEvent.ACTION_DRAG_STARTED -> {
                    return@OnDragListener true
                }
                DragEvent.ACTION_DROP -> {
                    addObject(event.x.toInt() - 100, event.y.toInt() - 100, event.clipData.getItemAt(0).text.toString())
                    return@OnDragListener true
                }
            }
            return@OnDragListener true
        }
    }

    override fun onSceneTouch(){
        unSelectActor()
        sceneEditView.setLayerVisible(currentLayerIndex)
    }

    private fun unSelectActor(){
        project.story?.let {
            it.scenes?.let {
                for(i in 0 until it[currentSceneIndex].layers[currentLayerIndex].actors.size){
                    it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect = false
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
    }

    override fun onLayerManagementDialog(layerNum: Int, position: Int) {
        val selectLayer = layerNum - 1
        project.story?.let {
            it.scenes?.let {
                var actor = it[currentSceneIndex].layers[selectLayer].actors[position]

        var popupWindow = PopupWindow(this)
        val popupView = LayoutInflater.from(this).inflate(R.layout.control_actor_dialog, null)
        popupView.duplicate.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    val actorData = Actor().apply {
                        positionX = actor.positionX + 100
                        positionY = actor.positionY + 100
                        positionZ = it[currentSceneIndex].layers[selectLayer].actors.size
                        sideLength = actor.sideLength
                        sideHeight = actor.sideHeight
                        resourcePath = actor.resourcePath
                        parentLayerIndex = selectLayer
                        isMovable = true
                    }
                    it[currentSceneIndex].layers[selectLayer].actors.add(actorData)
                    setActorPositionZ(it[currentSceneIndex].layers[selectLayer].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                    layerManagementDialog.setData(it[currentSceneIndex])
                }
            }
        }
        popupView.bring_to_front.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[selectLayer].actors.remove(actor)
                    it[currentSceneIndex].layers[selectLayer].actors.add(actor)
                    setActorPositionZ(it[selectLayer].layers[selectLayer].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                    layerManagementDialog.setData(it[currentSceneIndex])
                }
            }
        }
        popupView.send_to_back.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[selectLayer].actors.remove(actor)
                    it[currentSceneIndex].layers[selectLayer].actors.add(0, actor)
                    setActorPositionZ(it[currentSceneIndex].layers[selectLayer].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                    layerManagementDialog.setData(it[currentSceneIndex])
                }
            }
        }
        popupView.delete.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[selectLayer].actors.removeAt(actor.positionZ)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                    layerManagementDialog.setData(it[currentSceneIndex])
                }
            }
        }
        popupWindow.contentView = popupView
        popupWindow.width = ViewGroup.LayoutParams.WRAP_CONTENT
        popupWindow.height = ViewGroup.LayoutParams.WRAP_CONTENT
        popupWindow.isOutsideTouchable = true
        var valueXInPixels = resources.getDimension(R.dimen.actor_width)
        when(layerNum){
            1 -> popupWindow.showAsDropDown(layerManagementDialog.rv_layer_management_layer1, valueXInPixels.toInt() * position, 0)
            2 -> popupWindow.showAsDropDown(layerManagementDialog.rv_layer_management_layer2, valueXInPixels.toInt() * position, 0)
            3 -> popupWindow.showAsDropDown(layerManagementDialog.rv_layer_management_layer2, valueXInPixels.toInt() * position + valueXInPixels.toInt(),0)
            4 -> popupWindow.showAsDropDown(layerManagementDialog.rv_layer_management_layer1, valueXInPixels.toInt() * position, 0)
            5 -> popupWindow.showAsDropDown(layerManagementDialog.rv_layer_management_layer2, valueXInPixels.toInt() * position, 0)
        }
            }
        }
    }

    override fun onSetDialogText(dialogue: Actor, content: String) {
        project.story?.let {
            it.scenes?.let {
                dialogue.text = content
                it[currentSceneIndex].layers[currentLayerIndex].actors[dialogue.positionZ] = dialogue
                setActorPositionZ(it[currentSceneIndex].layers[currentLayerIndex].actors)
                sceneEditView.scene = it[currentSceneIndex]
                editTextDialog.dismiss()
            }
        }
    }

    override fun onEditTextDialogDismiss() {
        window.decorView.apply {
            systemUiVisibility = (View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_LOW_PROFILE
                    or View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)
        }
    }

    override fun onSelectBrush(brush: Brush?) {
        brush?.let {
            paint_view.brush = it
        }
    }
}
