package com.bomostory.sceneeditmodule.share

import android.graphics.PorterDuff
import android.graphics.drawable.Drawable
import android.media.MediaMetadataRetriever
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.support.v4.content.ContextCompat
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.SeekBar
import com.bomostory.sceneeditmodule.basicdata.Project
import com.bomostory.sceneeditmodule.utils.HorizontalItemDecoration
import com.bomostory.sceneeditmodule.utils.TimeUtils
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.dialog_share.view.*
import kotlinx.android.synthetic.main.movie_item_view.view.*
import java.io.File
import java.util.*

class ShareDialog : DialogFragment() {

    private val dialogTag = this::class.java.simpleName

    lateinit var file: File

    lateinit var project: Project

    var timer: Timer? = null

    var isPause = false

    var onClickEditCover = Runnable {}
    var onClickExportPdf = Runnable {}
    var onClickPrint = Runnable {}
    var onClickBomo = Runnable {}
    var onClickYoutube = Runnable {}
    var onClickFacebook = Runnable {}
    var onClickTwitter = Runnable {}
    var onClickMore = Runnable {}
    var onClickPlay = Runnable {}


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.dialog_share, container, false).apply {
            movieView_shareDialog_preview.scene = project.story?.scenes?.get(0)
            tv_shareDialog_name.text = project.name
            tv_shareDialog_author.text = project.author
            btn_shareDialog_editCover.setOnClickListener { onClickEditCover.run() }
            btn_shareDialog_exportPdf.setOnClickListener { onClickExportPdf.run() }
            btn_shareDialog_print.setOnClickListener { onClickPrint.run() }
            iv_shareDialog_bomo.setOnClickListener { onClickBomo.run() }
            iv_shareDialog_youtube.setOnClickListener { onClickYoutube.run() }
            iv_shareDialog_facebook.setOnClickListener { onClickFacebook.run() }
            iv_shareDialog_twitter.setOnClickListener { onClickTwitter.run() }
            iv_shareDialog_more.setOnClickListener { onClickMore.run() }

            try {
                val mediaMetadataRetriever = MediaMetadataRetriever()
                mediaMetadataRetriever.setDataSource(file.path)

                seekBarMoviePlay.apply {
                    max = mediaMetadataRetriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION).toInt()
                }

                videoShareDialogPreview.apply {
                    setZOrderOnTop(true)
                    setVideoPath(file.path)
                    setOnCompletionListener {
                        timer?.cancel()
                        timer?.purge()
                        timer = null

                        this@apply.iv_shareDialog_pause?.visibility = View.INVISIBLE
                        this@apply.iv_shareDialog_play?.visibility = View.VISIBLE
                        this@apply.seekBarMoviePlay?.progress = videoShareDialogPreview.currentPosition
                    }
                }

                iv_shareDialog_play.setOnClickListener {
                    it.visibility = View.INVISIBLE
                    iv_shareDialog_pause.visibility = View.VISIBLE

                    videoShareDialogPreview.start()
                    if (timer == null) {
                        timer = Timer()
                        timer?.scheduleAtFixedRate(object : TimerTask() {
                            override fun run() {
                                activity?.runOnUiThread {
                                    if (!isPause) {
                                        seekBarMoviePlay.progress = videoShareDialogPreview.currentPosition
                                        tv_shareDialog_time?.text = "${TimeUtils.getPlayMovieTimeFormat(seekBarMoviePlay.progress.toLong())}/${TimeUtils.getPlayMovieTimeFormat(seekBarMoviePlay.max.toLong())}"
                                    }
                                }
                            }
                        }, 0, 1)
                    } else {
                        isPause = false
                    }
                }

                iv_shareDialog_pause.setOnClickListener {
                    it.visibility = View.INVISIBLE
                    iv_shareDialog_play.visibility = View.VISIBLE
                    videoShareDialogPreview.pause()
                    isPause = true
                }

                tv_shareDialog_time?.text = "${TimeUtils.getPlayMovieTimeFormat(seekBarMoviePlay.progress.toLong())}/${TimeUtils.getPlayMovieTimeFormat(seekBarMoviePlay.max.toLong())}"

                seekBarMoviePlay.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
                    override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                        if (fromUser) {
                            videoShareDialogPreview.seekTo(progress)
                        }
                    }

                    override fun onStartTrackingTouch(seekBar: SeekBar?) {
                    }

                    override fun onStopTrackingTouch(seekBar: SeekBar?) {
                    }
                })
            } catch (e: Exception) {
                disableMovieView(this)
            }

            sceneList.apply {
                setHasFixedSize(true)
                addItemDecoration(HorizontalItemDecoration(resources.getDimensionPixelSize(R.dimen.horizontal_item_spacing)))
                layoutManager = LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false)
                adapter = MovieAdapter()
            }
        }
    }

    private fun disableMovieView(view: View) {
        view.apply {
            videoShareDialogPreview?.apply {
                isEnabled = false
            }
            seekBarMoviePlay?.apply {
                isEnabled = false
            }
            iv_shareDialog_play?.apply {
                isEnabled = false
                setColorFilter(ContextCompat.getColor(context, android.R.color.darker_gray), PorterDuff.Mode.SRC_ATOP)
            }
            iv_shareDialog_bomo?.apply {
                isEnabled = false
                setColorFilter(ContextCompat.getColor(context, android.R.color.darker_gray), PorterDuff.Mode.SRC_ATOP)
            }
            iv_shareDialog_more?.apply {
                isEnabled = false
                setColorFilter(ContextCompat.getColor(context, android.R.color.darker_gray), PorterDuff.Mode.SRC_ATOP)
            }
        }
    }

    fun show(manager: FragmentManager?) {
        manager?.apply {
            val ft = beginTransaction()
            findFragmentByTag(dialogTag)?.let { ft.remove(it) }
            ft.addToBackStack(null)
            ft.commit()
        }
        super.show(manager, dialogTag)
    }

    inner class MovieAdapter : RecyclerView.Adapter<MovieAdapter.ViewHolder>() {
        override fun onCreateViewHolder(p0: ViewGroup, p1: Int): ViewHolder {
            val view = LayoutInflater.from(p0.context).inflate(R.layout.scene_item_view, p0, false)
            return ViewHolder(view)
        }

        override fun getItemCount(): Int {
            project.story?.let {
                return it.scenes.size
            }
            return 0
        }

        override fun onBindViewHolder(p0: ViewHolder, p1: Int) {
            p0.bind(p1)
        }

        inner class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
            fun bind(p1: Int) {
                itemView.apply {
                    project.story?.apply {
                        screenshot.setImageDrawable(Drawable.createFromPath(this.scenes[p1].backgroundPath))
                    }
                }
            }
        }
    }
}